package dev.adriankuta.flights.ui.home

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dev.adriankuta.flights.core.util.Result
import dev.adriankuta.flights.core.util.asResult
import dev.adriankuta.flights.domain.search.ObserveAirportsUseCase
import dev.adriankuta.flights.domain.types.AirportInfo
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.combine
import kotlinx.coroutines.flow.stateIn
import javax.inject.Inject

@HiltViewModel
class HomeScreenViewModel @Inject constructor(
    private val observeAirportsUseCase: ObserveAirportsUseCase,
) : ViewModel() {

    private val selectedOriginAirport = MutableStateFlow<AirportInfo?>(null)
    private val selectedDestinationAirport = MutableStateFlow<AirportInfo?>(null)

    internal val uiState = homeUiState(
        useCase = observeAirportsUseCase,
        selectedOriginAirport = selectedOriginAirport,
        selectedDestinationAirport = selectedDestinationAirport,
    )
        .stateIn(
            scope = viewModelScope,
            started = SharingStarted.WhileSubscribed(5_000),
            initialValue = HomeUiState.Loading,
        )

    fun selectOriginAirport(airport: AirportInfo) {
        selectedOriginAirport.value = airport
        // If the selected destination is the same as the new origin, clear the destination
        if (selectedDestinationAirport.value?.code == airport.code) {
            selectedDestinationAirport.value = null
        }
    }

    fun selectDestinationAirport(airport: AirportInfo) {
        selectedDestinationAirport.value = airport
    }

    fun clearOriginAirport() {
        selectedOriginAirport.value = null
        // Clear destination as well since it depends on origin
        selectedDestinationAirport.value = null
    }

    fun clearDestinationAirport() {
        selectedDestinationAirport.value = null
    }
}

private fun homeUiState(
    useCase: ObserveAirportsUseCase,
    selectedOriginAirport: MutableStateFlow<AirportInfo?>,
    selectedDestinationAirport: MutableStateFlow<AirportInfo?>,
): Flow<HomeUiState> {
    return combine(
        useCase().asResult(),
        selectedOriginAirport,
        selectedDestinationAirport,
    ) { result, origin, destination ->
        when (result) {
            is Result.Error -> HomeUiState.Error(result.exception)
            is Result.Loading -> HomeUiState.Loading
            is Result.Success -> {
                val airports = result.data.orEmpty()
                HomeUiState.Success(
                    originAirports = airports,
                    destinationAirports = airports.filter { it.code != origin?.code },
                    selectedOriginAirport = origin,
                    selectedDestinationAirport = destination,
                )
            }
        }
    }
}

internal sealed interface HomeUiState {
    data object Loading : HomeUiState
    data class Success(
        val originAirports: List<AirportInfo>,
        val destinationAirports: List<AirportInfo>,
        val selectedOriginAirport: AirportInfo? = null,
        val selectedDestinationAirport: AirportInfo? = null,
    ) : HomeUiState

    data class Error(val exception: Throwable) : HomeUiState
}
